/********************************************************
 * Data Visualization - 2D                              *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Yan Kong                                     *
 * Supervisor: Rodger Luo                               *
 *                                                      *
 * Purpose: Demonstrate the popularity of Quentin       *
 *          Tatantino's movies, correlations between    *
 *          checkouts each year and Quentin's Oscar     *
 *          winning moments. Visualize it               *
 *          using 2D graph.                             *
 *                                                      *
 * Usage:                                               *
 *          Press '1', '2', '3', '4' to apply different *
 *          color schemes.                              *
 *          Press '0' to use gray scale.                *
 ********************************************************/

import java.util.*; 
import java.util.List; 
import java.lang.Math.*;

public class Triplet<T, U, V> {

    private final T first;
    private final U second;
    private final V third;

    public Triplet(T first, U second, V third) {
        this.first = first;
        this.second = second;
        this.third = third;
    }

    public T getFirst() { return first; }
    public U getSecond() { return second; }
    public V getThird() { return third; }
}

final int YEARS = 12;
final int ITEMTYPE = 0;
final int TITLE = 1;
final int COUNTS = 2;
final int STARTYEAR = 3;
final float marginToTop = 160;
final float marginToLeft = 40;
final float centerX = 640;
final float centerY = 360;
final int radius = 650;
final float LIFETIME = 10;
final int TEXTSIZE = 10;
final int WIDTH = 1280;
final int HEIGHT = 720;
ArrayList<float[]> allPoints = new ArrayList<float[]>();
boolean generated = false;
int lastStartIndex = -1;
float lifetime = LIFETIME;
float scannerSpeed = 0.05;
float currStartArc, currEndArc;
float[][] points;
Table totalCopies;
Table checkoutsYearly;
List<Triplet<String, String, Integer>> copies;
List<Triplet<String, String, int[]>> checkouts;
float maxCheckouts, minCheckouts = 1000;

//Only run one time
void setup() {
  frameRate(10);
  // setup the size of the window
  size(1280, 720); 
  // copies.csv format: itemType  title  Counts
  totalCopies = loadTable("QuentinTarantino_CopyCounts.csv", "header");
  // checkouts.csv format: bibNumber  itemType  title  Counts  2006  2007  2008  2009  2010  2011  2012  2013  2014  2015  2016  2017
  checkoutsYearly = loadTable("QuentinTarantino_CheckoutCounts.csv", "header");
  
  copies = new ArrayList();
  for(int i = 0; i < totalCopies.getRowCount(); i++)
  {  
    copies.add(new Triplet(totalCopies.getString(i, ITEMTYPE),totalCopies.getString(i, TITLE), totalCopies.getInt(i, COUNTS)));
    // println(copies.get(i).first, copies.get(i).second, copies.get(i).third);
  }
  
  checkouts = new ArrayList();
  for(int i = 0; i < checkoutsYearly.getRowCount(); i++)
  {
    int[] checkoutsForEachYear = new int[YEARS];
    for(int j = 0; j < YEARS; j++)
    {
      checkoutsForEachYear[j] = checkoutsYearly.getInt(i, j + STARTYEAR);
    }
    checkouts.add(new Triplet(checkoutsYearly.getString(i, ITEMTYPE), checkoutsYearly.getString(i, TITLE), checkoutsForEachYear));
    //print(checkouts.get(i).first, checkouts.get(i).second + " ");
    //for(int j = 0; j < checkouts.get(i).third.length; j++)
    //  print(checkouts.get(i).third[j] + " ");
    //print("\n");
  }
  
  // get max and min
  for(int i = 0; i < checkoutsYearly.getRowCount(); i++)
  {
    if(maxCheckouts < checkoutsYearly.getInt(i, COUNTS))
      maxCheckouts = checkoutsYearly.getInt(i, COUNTS);
    if(minCheckouts > checkoutsYearly.getInt(i, COUNTS))
      minCheckouts = checkoutsYearly.getInt(i, COUNTS);
  }
  points = new float[checkoutsYearly.getRowCount()][4];
  
  println("max:", maxCheckouts);
  println("min:", minCheckouts);
  //cellWidth = (width - marginToLeft * 2) / cols;
  //cellHeight = (height - marginToTop * 2) / rows;
  
  loadAllData();
  println("allPoints.size() = "+allPoints.size());
}

//Refresh the canvas every frame
void draw() {  
  background(0, 0, 0);
  // draw arc for copies
  DrawScanner();
  
  for(int i = 0 ; i<allPoints.size();i++){
    strokeWeight(map(allPoints.get(i)[2], minCheckouts, maxCheckouts, 1, 150));
    //println("currEndArc = "+currEndArc);
  
    SwitchFillColor(FindMovieIndex((int)allPoints.get(i)[4]), i);
     if(sin(currEndArc-allPoints.get(i)[3])>0)
    // SwitchFillColor(FindMovieIndex((int)allPoints.get(i)[4]));
    point(allPoints.get(i)[0],allPoints.get(i)[1]);
  }
  
  int startIndex = FindIndex(currStartArc);;
  int endIndex = FindIndex(currEndArc);
  DrawText(startIndex);
  DrawText(endIndex);
  //for(int i = 0; i < YEARS; i++)
  //  DrawText(i);
  
  DrawNote();
  // DrawText("Quentin Taratino", 0, 0);
  // #386330,#537F3A,#729B41,#96B847,#BFD44D,#EBF052
  // DrawRadar();
}